using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.IdentityModel.Selectors;

namespace Client
{
    public class Program
    {
        public static void Main(string[] args)
        {
            Console.WriteLine("Press any key when the service is ready.");
            Console.ReadKey(true);

            try
            {
                decimal result = 0;
                Uri mexUri = new Uri("http://localhost:8000/Derivatives/mex");
                ContractDescription contract = ContractDescription.GetContract(typeof(DerivativesCalculator.IDerivativesCalculator));

                EndpointAddress mexEndpointAddress = new EndpointAddress(mexUri);
                ServiceEndpointCollection endpoints = MetadataResolver.Resolve(contract.ContractType,
                      mexEndpointAddress);

                foreach (ServiceEndpoint endpoint in endpoints)
                {
                    if (endpoint.Contract.Namespace.Equals(contract.Namespace) &&
                          endpoint.Contract.Name.Equals(contract.Name))
                    {
                        ChannelFactory<DerivativesCalculator.IDerivativesCalculator> cf = new
                              ChannelFactory<DerivativesCalculator.IDerivativesCalculator>(endpoint.Binding, endpoint.Address);
                        cf.Credentials.ServiceCertificate.Authentication.RevocationMode =
                              System.Security.Cryptography.X509Certificates.X509RevocationMode.NoCheck;
                        DerivativesCalculator.IDerivativesCalculator chn = cf.CreateChannel();
                        result = chn.CalculateDerivative(
                            new string[] { "MSFT" },
                            new decimal[] { 3 },
                            new string[] { });
                        cf.Close();
                    }
                }
                Console.WriteLine(string.Format("Result: {0}", result));
            }
            catch (System.IdentityModel.Selectors.CardSpaceException cse)
            {
                Console.WriteLine("Generic Cardspace exception:" + cse.Message);
            }
            catch (IdentityValidationException)
            {
                Console.WriteLine("Recipient's certificate was not valid");
            }
            catch (System.IdentityModel.Selectors.UserCancellationException)
            {
                Console.WriteLine("User cancelled");
            }
            catch (UntrustedRecipientException)
            {
                Console.WriteLine("User does not trust the recipient");
            }
            catch (ServiceNotStartedException)
            {
                Console.WriteLine("Cardspace service not started");
            }
            catch (Exception e)
            {
                Console.WriteLine("Other exceptions :" + e.Message);
            }
            finally
            {
                Console.WriteLine("Press any key to exit.");
                Console.ReadKey(true);
            }
        }
    }
}
