using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.IdentityModel.Policy;
using RST = Microsoft.ServiceModel.Samples.SecurityTokenService.RequestSecurityToken;
using RSTR = Microsoft.ServiceModel.Samples.SecurityTokenService.RequestSecurityTokenResponse;

namespace Microsoft.ServiceModel.Samples.SecurityTokenService
{
    class CertificateAuthSTS : WsTrustResponder {}
    class SelfIssuedSamlAuthSTS : WsTrustResponder {}
    class UserNameAuthSTS : WsTrustResponder {}

    class WsTrustResponder : IWSTrustContract
    {
        public Message Cancel(Message request)
        {
            throw new FaultException("Action not implemented");
        }

        public Message Issue(Message request)
        {
            try
            {
                if (request == null)
                {
                    throw new ArgumentNullException("request");
                }

                // Parse the incoming request, an RST
                RST rst = new RST(request.GetReaderAtBodyContents());


                // SAMPLE EXTENSIBILITY TASK:
                // 
                // You could add code here to furthur check the validity of the incoming identity
                // by examining the claims in the AuthorizationContext
                // 
                AuthorizationContext ctx = OperationContext.Current.ServiceSecurityContext.AuthorizationContext;

                // Process the request and generate an RSTR
                RSTR rstr = new RSTR(rst);

                // Generate a response message
                Message response = Message.CreateMessage(MessageVersion.Default, Constants.WSTrust.Actions.IssueResponse, rstr);

                // Set the RelatesTo
                if ( request.Headers.MessageId != null )
                {
                    response.Headers.RelatesTo = request.Headers.MessageId;
                } 
                else 
                {
                    // not supported in this sample
                    throw new NotSupportedException("Caller must provide a Message Id");
                }

                // Send back to the caller
                return response;
            }
            catch (Exception e)
            {
                throw WSTrustFaultException.FromException(e);
            }
        }

        public Message Renew(Message request)
        {
            throw new FaultException<string>("Action Not Supported");
        }

        public Message Validate(Message request)
        {
            throw new FaultException<string>("Action Not Supported");
        }
    }
}
