sing System;
using System.Collections.Generic;
using System.Configuration;
using System.IdentityModel.Claims;
using System.IdentityModel.Policy;
using System.IO;
using System.ServiceModel;
using System.Web.Security;

namespace Service
{

    public class AccessChecker : ServiceAuthorizationManager
    {
        private Dictionary<string, string[]> accessRequirements = null;

        public AccessChecker()
        {
            this.accessRequirements = new Dictionary<string, string[]>();

            OperationRequirementsConfigurationSection 
                operationRequirementsConfigurationSection 
                = ConfigurationManager.GetSection("operationRequirements")
                as OperationRequirementsConfigurationSection;
            
            OperationRequirementsCollection requirements =
                operationRequirementsConfigurationSection.OperationRequirements;
            List<string> roles = null;
            foreach (OperationElement operationElement in requirements)
            {
                roles = new List<string>(operationElement.Roles.Count);
                foreach (RoleElement roleElement in operationElement.Roles)
                {
                    roles.Add(roleElement.Name);
                }

                this.accessRequirements.Add(
                    operationElement.Identifier,
                    roles.ToArray());
            }
            
        }

        public override bool CheckAccess(OperationContext operationContext)
        {
            string header =
            operationContext.RequestContext.RequestMessage.Headers.Action;
            string[] requiredRoles = null;
            if (!(this.accessRequirements.TryGetValue(header, out requiredRoles)))
            {
                return false;
            }

            string userName = null;
            foreach(ClaimSet claimSet in operationContext.ServiceSecurityContext.AuthorizationContext.ClaimSets)
            {
                foreach (Claim claim in claimSet)
                {
                    if (string.Compare(
                claim.ClaimType,
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name",
                true) == 0)
                    {
                        userName = (string)claim.Resource;
                        foreach (string requiredRole in requiredRoles)
                        {
                            if (Roles.Provider.IsUserInRole(userName, requiredRole))
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;

            

        }
    }
}