using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    public class MyCustomBindingElement: BindingElement
    {
        public MyCustomBindingElement()
        {
            Console.WriteLine("Constructing binding element.");
        }

        public MyCustomBindingElement(MyCustomBindingElement original)
        {
            Console.WriteLine("Copying binding element.");
        }

        public override BindingElement Clone()
        {
            Console.WriteLine("Cloning binding element.");
            return new MyCustomBindingElement(this);
        }

        public override T GetProperty<T>(BindingContext context)
        {
            return context.GetInnerProperty<T>();
        }

        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            Console.WriteLine("Querying if the binding element can build a channel factory of type {0}.", typeof(TChannel).Name);
            if (typeof(TChannel) == typeof(IRequestChannel))
            {
                return context.CanBuildInnerChannelFactory<TChannel>();
            }
            else
            {
                return false;
            }
        }

        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            Console.WriteLine("Querying if the binding element can build a listener of type {0}.", typeof(TChannel).Name);
            if (typeof(TChannel) == typeof(IReplyChannel))
            {
                return context.CanBuildInnerChannelListener<TChannel>();
            }
            else
            {
                return false;
            }
        }

        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            Console.WriteLine("Asking the binding element for a channel factory of type {0}.", typeof(TChannel).Name);
            return (IChannelFactory<TChannel>)(object)new MyCustomChannelFactory<TChannel>(this,context);
        }

        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            Console.WriteLine("Asking the binding element for a listener of type {0}.", typeof(TChannel).Name);
            return(IChannelListener<TChannel>)(object)new MyCustomChannelListener<TChannel>(this,context);
        }
    }
}
