using System;
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomChannelListener<TChannel>: ChannelListenerBase<IReplyChannel> where TChannel: class, IChannel
    {
        IChannelListener<TChannel> innerChannelListener = null;

        public MyCustomChannelListener(MyCustomBindingElement bindingElement, BindingContext context)
            : base(context.Binding)
        {
            Console.WriteLine("Constructing the channel listener.");
            this.innerChannelListener = context.BuildInnerChannelListener<TChannel>();
            if (this.innerChannelListener == null)
            {
                throw new InvalidOperationException("MyCustomChannelListener requires an inner IChannelFactory.");
            }

        }

        





        protected override IReplyChannel OnAcceptChannel(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginAcceptChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            Console.WriteLine("Channel listener BeginAcceptChannel.");
            return this.innerChannelListener.BeginAcceptChannel(timeout, callback, state);
        }

        protected override IReplyChannel OnEndAcceptChannel(IAsyncResult result)
        {
            Console.WriteLine("Channel listener EndAcceptChannel.");
            IReplyChannel replyChannel = ((IReplyChannel)this.innerChannelListener.EndAcceptChannel(result));
            if (replyChannel == null)
            {
                return null;
            }
            return new MyCustomReplyChannel(replyChannel);
        }

        protected override IAsyncResult OnBeginWaitForChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override bool OnEndWaitForChannel(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override bool OnWaitForChannel(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override Uri Uri
        {
            get 
            {
                Console.WriteLine("Retrieving channel listener URI.");
                return this.innerChannelListener.Uri;
            }
        }

        protected override void OnAbort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnClose(TimeSpan timeout)
        {
            Console.WriteLine("Channel listener OnClose.");
            this.innerChannelListener.Close(timeout);
        }

        protected override void OnEndClose(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnEndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnOpen(TimeSpan timeout)
        {
            Console.WriteLine("Channel listener OnOpen.");
            this.innerChannelListener.Open(timeout);
           
        }
    }
}
