using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomChannelListener<TChannel> : ChannelListenerBase<IDuplexSessionChannel> where TChannel : class, IChannel
    {
        private Uri uri = null;
        private MessageEncoderFactory encoderFactory = null;
        private TcpListener tcpListener = null;
        private BufferManager bufferManager = null;
        private TransportBindingElement bindingElement = null;
        private CommunicationState state = CommunicationState.Created;
        
        
        public MyCustomChannelListener(TransportBindingElement bindingElement, BindingContext context)
            : base(context.Binding)
        {
            this.bindingElement = bindingElement;

            this.bufferManager = BufferManager.CreateBufferManager(bindingElement.MaxBufferPoolSize, (int)bindingElement.MaxReceivedMessageSize);

            MessageEncodingBindingElement encodingBindingElement = context.BindingParameters.Remove<MessageEncodingBindingElement>();
            if (encodingBindingElement != null)
            {
                this.encoderFactory = encodingBindingElement.CreateMessageEncoderFactory();
            }
            else
            {
                this.encoderFactory = new MyCustomMessageEncoderBindingElement().CreateMessageEncoderFactory();
            }

            this.uri = new Uri(context.ListenUriBaseAddress, context.ListenUriRelativeAddress);

        }
        
        protected override IDuplexSessionChannel OnAcceptChannel(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginAcceptChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            return this.tcpListener.BeginAcceptTcpClient(callback, state);
            
        }

        protected override IDuplexSessionChannel OnEndAcceptChannel(IAsyncResult result)
        {
            if (this.state != CommunicationState.Closed)
            {
                TcpClient client = this.tcpListener.EndAcceptTcpClient(result);
                return new MyCustomDuplexSessionChannel(this.uri, this.bindingElement, this.encoderFactory, this.bufferManager, client);
            }
            else
            {
                return null;
            }
            

        }

        protected override IAsyncResult OnBeginWaitForChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override bool OnEndWaitForChannel(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override bool OnWaitForChannel(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override Uri Uri
        {
            get 
            {
                return this.uri;
            }
        }

        protected override void OnAbort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnClose(TimeSpan timeout)
        {
            this.state = CommunicationState.Closed;
            this.tcpListener.Stop();
        }

        protected override void OnEndClose(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnEndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnOpen(TimeSpan timeout)
        {
            if (uri == null)
            {
                throw new InvalidOperationException("Some address is required before this Channel Listener can be used");
            }

            IPEndPoint ipEndpoint = new IPEndPoint(IPAddress.Parse(uri.Host), uri.Port);
            this.tcpListener = new TcpListener(ipEndpoint);
            this.tcpListener.Start();


            
        }
    }
}