using System;
using System.Collections.Generic;
using System.Drawing;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
    [DataContract]
    public struct QuizResponse
    {
        [DataMember]
        public Student Responder;
        [DataMember]
        public QuizItem Item;
        [DataMember]
        public string ResponseText;
        [DataMember]
        public string ResponseIdentifier;
        [DataMember]
        public bool? Correct;
        public bool Submitted;
    }

    [DataContract]
    public struct QuizItem
    {
        [DataMember]
        public string Identifier;
        [DataMember]
        public byte[] ItemImage;
        [DataMember]
        public string Text;
        [DataMember]
        public QuizResponse[] Responses;
        public bool Submitted;

        public string ResponseText
        {
            get
            {
                foreach (QuizResponse response in Responses)
                {
                    if (response.Submitted)
                    {
                        return response.ResponseText;
                    }
                }

                return null;
            }
        }

        public bool? Correct
        {
            get
            {
                foreach (QuizResponse response in Responses)
                {
                    if (response.Submitted)
                    {
                        return response.Correct;
                    }
                }

                return null;
            }
        }
    }

    [DataContract]
    public struct Student
    {
        [DataMember]
        public string Name;
        [DataMember]
        public byte[] Image;
    }





    [ServiceContract(CallbackContract = typeof(IQuizQuestion))]
    public interface IQuizQuestion
    {
        [OperationContract(IsOneWay = true)]
        void AddItem(QuizItem item);
    }

    [ServiceContract(CallbackContract = typeof(IQuizResponse))]
    public interface IQuizResponse
    {
        [OperationContract(IsOneWay = true)]
        void SendResponse(QuizResponse response);
    }

    [ServiceContract(CallbackContract = typeof(IQuizManagement))]
    public interface IQuizManagement
    {
        [OperationContract(IsOneWay = true)]
        void Join(Student student);

        [OperationContract(IsOneWay = true)]
        void Announce(string announcement);
    }

    public class QuizQuestionClient : DuplexClientBase<IQuizQuestion>, IQuizQuestion
    {
        public QuizQuestionClient(InstanceContext callbackInstance, string endpointConfigurationName)
            : base(callbackInstance, endpointConfigurationName)
        {
        }

        #region IQuizQuestion Members

        public void AddItem(QuizItem item)
        {
            base.Channel.AddItem(item);
        }

        #endregion
    }

    public class QuizResponseClient : DuplexClientBase<IQuizResponse>, IQuizResponse
    {
        public QuizResponseClient(InstanceContext callbackInstance, Binding binding, EndpointAddress address)
            : base(callbackInstance, binding, address)
        {
        }
        
        #region IQuizResponse Members

        public void SendResponse(QuizResponse response)
        {
            base.Channel.SendResponse(response);
        }

        #endregion
    }

    public class QuizManagementClient : DuplexClientBase<IQuizManagement>, IQuizManagement
    {
        public QuizManagementClient(InstanceContext callbackInstance, string endpointConfigurationName)
            : base(callbackInstance,endpointConfigurationName)
        {
        }

        #region IQuizManagement Members

        public void Join(Student student)
        {
            base.Channel.Join(student);
        }

        public void Announce(string announcement)
        {
            base.Channel.Announce(announcement);
        }

        #endregion
    }


    


}
